import os

from .html import HTMLFile
from .utils import create_parent_directory


class RedirectFile(HTMLFile):
    def render_to_output(self, output_path, *args, **kwargs):
        target_path = os.path.join(output_path, self.output_relative_path)
        create_parent_directory(target_path)

        redirect_to = open(self.path, 'r').read().strip()

        template = kwargs['jinja_environment'].get_template('redirect.template.html')
        with open(target_path, 'w+') as f:
            f.write(template.render(redirect_url=redirect_to))



import os
import glob

from bs4 import BeautifulSoup

from .core import Check, Message
from .registry import register


@register
class PageTitlesCheck(Check):
    def run(self):
        messages = []
        page_titles = {}

        ignored_titles = [
            'Redirecting...',
        ]

        for path in glob.iglob(os.path.join(self.combine.output_path, '**', '*.html'), recursive=True):
            with open(path, 'r') as f:
                soup = BeautifulSoup(f, 'html.parser')
                title = soup.title
                if title:
                    if title.string in ignored_titles:
                        continue

                    if title.string in page_titles:
                        page_titles[title.string].append(path)
                    else:
                        page_titles[title.string] = [path]
                else:
                    messages.append(Message('Page title is missing', 'seo.E001', hint='Add a <title> in <head>', object=path))

        for title, paths in page_titles.items():
            if len(paths) > 1:
                messages.append(Message('Duplicate page title', 'seo.W001', hint='Change the <title> attribute', object=paths))

            if len(title) > 60:
                messages.append(Message('Long page title', 'seo.W002', hint='Make the title shorter. https://moz.com/learn/seo/title-tag', object=paths))

        # TODO put brand at the end? Check if lots of titles start with the same thing and warn to move it to the end...

        return messages


@register
class AltTextCheck(Check):
    def run(self):
        messages = []

        for path in glob.iglob(os.path.join(self.combine.output_path, '**', '*.html'), recursive=True):
            with open(path, 'r') as f:
                soup = BeautifulSoup(f, 'html.parser')
                for img in soup.findAll('img'):
                    if not img.attrs.get('alt', ''):
                        messages.append(Message('Alt text missing on img', 'seo.E002', hint=f'Add alt attribute to img tag', object=(img, path)))

        return messages



from jinja2 import nodes
from jinja2.ext import Extension

import markdown


class MarkdownExtension(Extension):
    tags = set(['markdown'])

    def __init__(self, environment):
        super().__init__(environment)

    def parse(self, parser):
        lineno = next(parser.stream).lineno
        body = parser.parse_statements(['name:endmarkdown'], drop_needle=True)
        return nodes.CallBlock(self.call_method('_support'),
                               [], [], body).set_lineno(lineno)

    def _support(self, caller):
        """Helper callback."""
        markdown_content = caller()
        html_content = markdown.markdown(markdown_content)
        return html_content



import os

from .core import File
from .ignored import IgnoredFile
from .template import TemplateFile
from .html import HTMLFile
from .sass import SassFile
from .redirect import RedirectFile


def file_class_for_path(path):
    base, ext = os.path.splitext(path)

    if os.path.basename(path).startswith('_') or os.path.basename(path).startswith('.'):
        return IgnoredFile

    secondary_extension_classes = {
        '.template': TemplateFile,
    }

    base2, ext2 = os.path.splitext(base)

    if ext2 in secondary_extension_classes:
        return secondary_extension_classes[ext2]

    classes = {
        '.html': HTMLFile,
        '.sass': SassFile,
        '.scss': SassFile,
        '.redirect': RedirectFile,
    }

    if ext in classes:
        return classes[ext]

    return File
